#ifndef _PPC_IO_TIVO_H
#define _PPC_IO_TIVO_H

#include <linux/config.h>
#include <linux/ioctl.h>
#include <asm/page.h>

#ifdef CONFIG_TCD

#define TCD_IO_BASE_FLASH		0xffe00000
#define TCD_IO_SIZE_FLASH		(2 * 1024 * 1024)
#define TCD_IO_BASE_MEDIASWITCH		0xffd00000
#define TCD_IO_SIZE_MEDIASWITCH		(1 * 1024 * 1024)
#define TCD_IO_BASE_UART		0xffb00000
#define TCD_IO_BASE_DSSUART		(TCD_IO_BASE_UART + (256 * 1024))
#define TCD_IO_BASE_UART2		(TCD_IO_BASE_UART + (512 * 1024))
#define TCD_IO_SIZE_UART		(1 * 1024 * 1024)
#define TCD_IO_BASE_MODEM		0xffc00000
#define TCD_IO_SIZE_MODEM		(1 * 1024 * 1024)
#define TCD_IO_BASE_INPUT		0xfc000000
#define TCD_IO_SIZE_INPUT		(32 * 1024 * 1024)
#define TCD_IO_BASE_8390		0xf4000000
#define TCD_IO_SIZE_8390		(1 * 1024 * 1024)
#define TCD_IO_BASE_SERIAL		0x40000000
#define TCD_IO_SIZE_SERIAL		256

#endif

#define _IOBASE_OTHER_			0xffb00000
#define IOBASE_FLASH			TCD_IO_BASE_FLASH
#define IOBASE_MEDIASWITCH		TCD_IO_BASE_MEDIASWITCH
#define IOBASE_UART			TCD_IO_BASE_UART
#define IOBASE_DSSUART			TCD_IO_BASE_DSSUART
#define IOBASE_UART2			TCD_IO_BASE_UART2
#define IOBASE_MODEM			TCD_IO_BASE_MODEM
#define IOBASE_INPUT			TCD_IO_BASE_INPUT
#define IOBASE_UART			TCD_IO_BASE_UART
#define IOBASE_8390			TCD_IO_BASE_8390
#define IOBASE_SERIAL			0xf4100000

/*
 * IOCTL commands for talking to the contiguous memory 
 * manager implemented as part of /dev/mem.
 * (Well, not just the contiguous memory anymore ...)
 */
#ifdef CONFIG_CONTIGMEM
#define CONTIGMEM_IOC_MAGIC		'N'
#define CONTIGMEM_GETADDR		_IOR(CONTIGMEM_IOC_MAGIC, 1, 0)
#define CONTIGMEM_GETSIZE		_IOR(CONTIGMEM_IOC_MAGIC, 2, 0)
#define SETARG_PTRS             	_IOW(CONTIGMEM_IOC_MAGIC, 3, 0)

#ifdef __KERNEL__
#define CONTIGMEM_SIZE			(1 * 1024 * 1024)
#define CONTIGMEM_MINLEFT		(4 * 1024 * 1024)
extern unsigned long contigmem_start;
extern unsigned long mem_get_contigmem_start(void);
extern unsigned long mem_get_contigmem_size(void);
#endif
#endif

static inline unsigned char __inb__(volatile unsigned char *addr) 
{ asm volatile("eieio" : :); return(*(volatile unsigned char *)addr); }

static inline unsigned short __inw__(volatile unsigned short *addr) 
{ asm volatile("eieio" : :); return(*(volatile unsigned short *)addr); }

static inline unsigned int __inl__(volatile unsigned int *addr) 
{ asm volatile("eieio" : :); return(*(volatile unsigned int *)addr); }

static inline void __outb__(char b, volatile unsigned char *addr) 
{ asm volatile("eieio" : :); *(volatile unsigned char *)addr = b; }

static inline void __outw__(short b, volatile unsigned short *addr) 
{ asm volatile("eieio" : :); *(volatile unsigned short *)addr = b; }

static inline void __outl__(int b, volatile unsigned int *addr) 
{ asm volatile("eieio" : :); *(volatile unsigned int *)addr = b; }

#define inb(addr) __inb__((volatile unsigned char *) (addr))
#define inw(addr) __inw__((volatile unsigned short *) (addr))
#define inl(addr) __inl__((volatile unsigned int *) (addr))
#define outb(b,addr) __outb__((b), (volatile unsigned char *) (addr))
#define outw(b,addr) __outw__((b), (volatile unsigned short *) (addr))
#define outl(b,addr) __outl__((b), (volatile unsigned int *) (addr))

#define readb(addr) __inb__((volatile unsigned char *) (addr))
#define readw(addr) __inw__((volatile unsigned short *) (addr))
#define readl(addr) __inl__((volatile unsigned int *) (addr))
#define writeb(b,addr) __outb__((b), (volatile unsigned char *) (addr))
#define writew(b,addr) __outw__((b), (volatile unsigned short *) (addr))
#define writel(b,addr) __outl__((b), (volatile unsigned int *) (addr))

#define inb_p(port) (__inb__((volatile unsigned char *)(port)))
#define inw_p(port) (__inw__((volatile unsigned short *)(port)))
#define inl_p(port) (__inl__((volatile unsigned long *)(port)))
#define outb_p(val, port) (__outb__((val), (volatile unsigned char *)(port)))
#define outw_p(val, port) (__outw__((val), (volatile unsigned short *)(port)))
#define outl_p(val, port) (__outl__((val), (volatile unsigned long *)(port)))

extern void _insb(volatile unsigned char *port, void *buf, int ns);
extern void _outsb(volatile unsigned char *port, const void *buf, int ns);
extern void _insw(volatile unsigned short *port, void *buf, int ns);
extern void _outsw(volatile unsigned short *port, const void *buf, int ns);
extern void _insl(volatile unsigned long *port, void *buf, int nl);
extern void _outsl(volatile unsigned long *port, const void *buf, int nl);

extern void _insw_ns(volatile unsigned short *port, void *buf, int ns);
extern void _outsw_ns(volatile unsigned short *port, const void *buf, int ns);
extern void _insl_ns(volatile unsigned long *port, void *buf, int nl);
extern void _outsl_ns(volatile unsigned long *port, const void *buf, int nl);

#ifdef __KERNEL__

#define insb(port, buf, len)	_insb((volatile unsigned char *)(port), buf, len)
#define insw(port, buf, len)	_insw((volatile unsigned short *)(port), buf, len)
#define insl(port, buf, len)	_insl((volatile unsigned long *)(port), buf, len)
#define outsb(port, buf, len)	_outsb((volatile unsigned char *)(port), buf, len)
#define outsw(port, buf, len)	_outsw((volatile unsigned short *)(port), buf, len)
#define outsl(port, buf, len)	_outsl((volatile unsigned long *)(port), buf, len)

#define virt_to_bus(addr)	(addr)
#define bus_to_virt(addr)	(addr)
#define virt_to_phys(addr)	(addr)
#define phys_to_virt(addr)	(addr)

/* returns 0 if address OK, -1 otherwise */
extern int io_touch(unsigned long addr);

/*
 * Map in an area of physical address space, for accessing
 * I/O devices etc.
 */
extern inline void *ioremap(unsigned long address, unsigned long size) {
    return (void *) address;
}
extern inline void *__ioremap(unsigned long address, unsigned long size) {
    return (void *) address;
}
extern inline void iounmap(void *addr) { }
#endif /* __KERNEL__ */

#endif
