/*
  Public definitions for the Giant Slalom SCSI driver
*/

#include <linux/ioctl.h>

typedef struct {
    unsigned long  address;
    unsigned long  length;
    void          *data;
} GSIODescriptor;

typedef struct {
    unsigned long  length;
    void          *data;
} GSPLDDescriptor;

typedef struct {
  unsigned long address;
  unsigned short mask;
  unsigned short bits;
} GSMaskDescriptor;

#define kPLDMaxImageSize 0x10000

// ioctl selectors

#define     kGSReadBridge        _IOW('G', 0, GSIODescriptor)
#define     kGSWriteBridge       _IOW('G', 1, GSIODescriptor)
#define     kGSReadWindow        _IOW('G', 2, GSIODescriptor)
#define     kGSWriteWindow       _IOW('G', 3, GSIODescriptor)
#define     kGSDownloadPLD       _IOW('G', 4, GSPLDDescriptor)
#define     kGSInitDone          _IO('G', 5)
#define     kGSSetDebugMode      _IOR('G', 6, unsigned long)
#define     kGSResetFIFOs        _IO('G', 7)
#define     kGSReserve           _IO('G', 8)
#define     kGSRelease           _IO('G', 9)
#define     kGSReadFifoReset     _IO('G', 10)
#define     kGSReportDiagMode    _IOR('G', 11, unsigned long)
#define     kGSWriteBridgeMasked _IOW('G', 12, GSMaskDescriptor)

// Exported functions for other drivers to use

#ifdef __KERNEL__

int xmit_gs_spi (unsigned char *data, int len, void (*callback)(int));

int GSReadBridge(unsigned long address, unsigned long length, void *data );
int GSWriteBridge(unsigned long address, unsigned long length, void *data );
int GSWriteBridgeMasked(unsigned long address, unsigned short mask, unsigned short bits );

#define kGSBControlRegAddr (unsigned long) 0x00000000

#define kGSBControlRegSpdifInputMask (unsigned short) 0xFFBF
#define kGSBControlRegSpdifInput7020  (unsigned short) 0x0000
#define kGSBControlRegSpdifInputCDDA  (unsigned short) 0x0040

#define kGSBControlRegAudioClkMask (unsigned short) 0xFE7F
#define kGSBControlRegAudioClk96KHz  (unsigned short) 0x0000
#define kGSBControlRegAudioClk441KHz  (unsigned short) 0x0080
#define kGSBControlRegAudioClk32KHz  (unsigned short) 0x0100
#define kGSBControlRegAudioClk48KHz  (unsigned short) 0x0180

#define kGSBControlRegAudioOutMask (unsigned short) 0xFDFF
#define kGSBControlRegAudioOutMute (unsigned short) 0x0000
#define kGSBControlRegAudioOutUnmute (unsigned short) 0x0200

#endif


